#!/usr/bin/env python
# -*- coding: utf-8 -*-
import email
import smtpd
import asyncore
import logging
import socket
import base64

from lathermail.compat import bytes

log = logging.getLogger(__name__)


class SMTPChannelWithAuth(smtpd.SMTPChannel, object):

    def __init__(self, server, conn, addr, on_close=lambda s: None):
        smtpd.SMTPChannel.__init__(self, server, conn, addr)
        self.user = None
        self.password = None
        self.fqdn = socket.getfqdn()
        self._addr = addr
        self._on_close = on_close

    def close(self):
        smtpd.SMTPChannel.close(self)
        try:
            self._on_close(self._addr)
        except:
            log.exception("Exception in on_close for %s:", self._addr)

    def smtp_EHLO(self, arg):
        self.push("250-%s\r\n"
                  "250 AUTH PLAIN" % self.fqdn)
        self.seen_greeting = arg

    def smtp_AUTH(self, arg):
        try:
            user, password = self.decode_plain_auth(arg)
        except ValueError:
            self.push("535 5.7.8  Authentication credentials invalid")
        else:
            self.push("235 2.7.0  Authentication Succeeded")
            self.user, self.password = user.decode("ascii"), password.decode("ascii")

    def smtp_MAIL(self, arg):
        if not (self.user and self.password):
            self.push("530 5.7.0  Authentication required")
            return
        smtpd.SMTPChannel.smtp_MAIL(self, arg)

    @staticmethod
    def decode_plain_auth(arg):
        user_password = arg.split()[-1]
        user_password = base64.b64decode(bytes(user_password, "ascii"))
        return user_password.split(b"\0", 3)[1:]


class InboxServer(smtpd.SMTPServer, object):

    def __init__(self, localaddr, handler):
        super(InboxServer, self).__init__(localaddr, None)
        self._handler = handler
        self._channel_by_peer = {}

    def handle_accept(self):
        pair = self.accept()
        if pair is not None:
            conn, addr = pair
            log.info("Incoming connection from %s", repr(addr))
            # Since there is no way to pass channel or additional data (user, password) to process_message()
            # we have dict of channels indexed by peers. It's cleaned on channel close()
            self._channel_by_peer[addr] = SMTPChannelWithAuth(self, conn, addr, on_close=self._on_channel_close)

    def process_message(self, peer, mailfrom, rcpttos, data):
        channel = self._channel_by_peer.get(peer)
        if not channel:
            log.error("No channel found for peer %s. Channels dump: %s", peer, self._channel_by_peer)
            return "451 Internal confusion"

        log.info("Storing message: inbox: '%s', from: %r, to: %r, peer: %r",
                 channel.user, mailfrom, rcpttos, peer)
        message = email.message_from_string(data)
        return self._handler(to=rcpttos, sender=mailfrom, message=message, body=data,
                             user=channel.user, password=channel.password)

    def _on_channel_close(self, peer):
        self._channel_by_peer.pop(peer, None)
        log.info("Remove channel for peer: %s. Active channels: %s", peer, len(self._channel_by_peer))


def serve_smtp(host="127.0.0.1", port=10252, handler=None):
    if handler is None:

        def handler(to, sender, *args):
            print("{0} <- {1}".format(to, sender))

    InboxServer((host, port), handler)
    log.info("Running SMTP server on %s:%s", host, port)
    while True:
        try:
            asyncore.loop(use_poll=True, count=10)
        except:
            log.exception("Error in loop:")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging


engine = None


def init(app):
    global engine

    db_uri = app.config["DB_URI"]
    if db_uri.startswith("mongodb:/"):
        app.config["MONGO_URI"] = db_uri
        from .storage import mongo
        engine = mongo
    else:
        app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
        from .storage import alchemy
        engine = alchemy

    logging.info("Using '%s' DB engine. URI: '%s'", engine.__name__, db_uri)
    engine.init_app_for_db(app)



TEXT_FIELDS = {
    "sender.name", "sender.address", "recipients.name", "recipients.address",
    "subject",
}
SUFFIX_CONTAINS = "_contains"

CONTAINS_FIELDS = {field + SUFFIX_CONTAINS for field in TEXT_FIELDS}

ALLOWED_QUERY_FIELDS = {"_id", "read"} | TEXT_FIELDS | CONTAINS_FIELDS



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import email
from email.header import decode_header
from email.utils import getaddresses

from .compat import bytes, IS_PY3, unicode


def convert_addresses(raw_header):
    result = []
    name_addr_pairs = getaddresses([raw_header])
    for name, addr in name_addr_pairs:

        result.append({"name": _header_to_unicode(name), "address": addr})
    return result


def convert_message_to_dict(to, sender, message, body, user, password):
    from_addr = message.get("From") or sender
    to = message.get("To") or ",".join(to)
    subject = message.get("Subject") or ""

    result = {
        "inbox": user,
        "password": password,
        "message_raw": bytes(body, "utf8"),
        "sender_raw": from_addr,
        "recipients_raw": to,
        # for easy searching
        "sender": convert_addresses(from_addr)[0],
        "recipients": convert_addresses(to),
        "subject": _header_to_unicode(subject),
        "read": False,
    }
    return result


def expand_message_fields(message_info, include_attachment_bodies=False):
    raw = message_info["message_raw"]
    if IS_PY3:
        raw = raw.decode("utf8")
    message = email.message_from_string(raw)
    message_info["parts"] = list(_iter_parts(message, include_attachment_bodies))
    return message_info


def _iter_parts(message, include_attachment_bodies):
    parts = [message] if not message.is_multipart() else message.walk()

    index = 0
    for part in parts:
        filename = part.get_filename() or part.get("content-id")
        is_attachment = filename is not None
        if not include_attachment_bodies and is_attachment:
            body = None
        else:
            body = part.get_payload(decode=True)
        if not is_attachment and not body:
            continue

        charset = part.get_content_charset()
        if charset:
            try:
                body = body.decode(charset)
            except Exception:
                pass

        yield {
            "index": index,
            "type": part.get_content_type(),
            "is_attachment": is_attachment,
            "filename": _header_to_unicode(filename) if filename else None,
            "charset": charset,
            "body": body,
            "size": len(body) if body else 0
        }
        index += 1


def _header_to_unicode(header):
    data, encoding = decode_header(header)[0]
    if encoding:
        data = data.decode(encoding)
    return data



import os


DB_URI = "sqlite:///" + os.path.expanduser("~/.lathermail.db")
DEBUG_MODE = False
SMTP_HOST = "127.0.0.1"
SMTP_PORT = 2525
API_HOST = "127.0.0.1"
API_PORT = 5000
SQLITE_FAST_SAVE = True
SQLALCHEMY_TRACK_MODIFICATIONS = False



import sys

IS_PY3 = sys.version_info[0] == 3

if IS_PY3:
    from http.client import NO_CONTENT
    from email import encoders as Encoders
    from urllib.parse import quote, urlencode
    unicode = str
    bytes = bytes
else:
    from email import Encoders
    from httplib import NO_CONTENT
    from urllib import quote, urlencode
    unicode = unicode
    _orig_bytes = bytes
    bytes = lambda s, *a: _orig_bytes(s)
